@@ -1,6 +1,7 @@
 #include <tiffio.h>
 #include <string.h>
 #include <stdio.h>
+#include <errno.h>
 #include "imtiff.h"
 #include "imext.h"
 
@@ -283,9 +284,28 @@ tiffio_context_init(tiffio_context_t *c, io_glue *ig);
 static void
 tiffio_context_final(tiffio_context_t *c);
 
-static toff_t sizeproc(thandle_t x) {
-  (void)x;
-	return 0;
+static toff_t
+sizeproc(thandle_t h) {
+  io_glue *ig = ((tiffio_context_t *)h)->ig;
+
+  /* iolayer doesn't have a size callback, use seek() to find
+     the end
+  */
+  toff_t orig_off = i_io_seek(ig, 0, SEEK_CUR);
+  if (orig_off < 0) {
+    i_push_error(errno, "seek to current failed");
+    return -1;
+  }
+  off_t size = i_io_seek(ig, 0, SEEK_END);
+  if (size < 0) {
+    i_push_error(errno, "seek to end failed");
+    return -1;
+  }
+  if (i_io_seek(ig, orig_off, SEEK_SET) < 0) {
+    i_push_error(errno, "seek restore failed");
+    return -1;
+  }
+  return size;
 }
 
 
